@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.5.2.5 2007/09/14 11:06:35 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.5.2.7 2007/10/12 04:32:13 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -63,6 +63,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 static void cancel_request(CancelPacket *sp, int secondary_backend);
 static RETSIGTYPE die(int sig);
 static RETSIGTYPE close_idle_connection(int sig);
+static RETSIGTYPE authentication_timeout(int sig);
 static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 static void send_frontend_exits(void);
 static int s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password);
@@ -871,6 +872,12 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 		return NULL;
 	}
 
+	if (pool_config->authentication_timeout > 0)
+	{
+		pool_signal(SIGALRM, authentication_timeout);
+		alarm(pool_config->authentication_timeout);
+	}
+
 	/* read startup packet length */
 	if (pool_read(cp, &len, sizeof(len)))
 	{
@@ -883,12 +890,22 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	{
 		pool_error("read_startup_packet: incorrect packet length (%d)", len);
 	}
+	else if (len >= MAX_STARTUP_PACKET_LENGTH)
+	{
+		pool_error("read_startup_packet: invalid startup packet");
+		pool_free_startup_packet(sp);
+		alarm(0);
+		pool_signal(SIGALRM, SIG_IGN);
+		return NULL;
+	}
 
 	sp->startup_packet = calloc(len, 1);
 	if (!sp->startup_packet)
 	{
 		pool_error("read_startup_packet: out of memory");
 		pool_free_startup_packet(sp);
+		alarm(0);
+		pool_signal(SIGALRM, SIG_IGN);
 		return NULL;
 	}
 
@@ -896,6 +913,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	if (pool_read(cp, sp->startup_packet, len))
 	{
 		pool_free_startup_packet(sp);
+		alarm(0);
+		pool_signal(SIGALRM, SIG_IGN);
 		return NULL;
 	}
 
@@ -915,6 +934,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			{
 				pool_error("read_startup_packet: out of memory");
 				pool_free_startup_packet(sp);
+				alarm(0);
+				pool_signal(SIGALRM, SIG_IGN);
 				return NULL;
 			}
 			strncpy(sp->database, sp2->database, SM_DATABASE);
@@ -924,6 +945,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			{
 				pool_error("read_startup_packet: out of memory");
 				pool_free_startup_packet(sp);
+				alarm(0);
+				pool_signal(SIGALRM, SIG_IGN);
 				return NULL;
 			}
 			strncpy(sp->user, sp2->user, SM_USER);
@@ -943,6 +966,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 					{
 						pool_error("read_startup_packet: out of memory");
 						pool_free_startup_packet(sp);
+						alarm(0);
+						pool_signal(SIGALRM, SIG_IGN);
 						return NULL;
 					}
 				}
@@ -954,6 +979,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 					{
 						pool_error("read_startup_packet: out of memory");
 						pool_free_startup_packet(sp);
+						alarm(0);
+						pool_signal(SIGALRM, SIG_IGN);
 						return NULL;
 					}
 				}
@@ -968,6 +995,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			{
 				pool_error("read_startup_packet: out of memory");
 				pool_free_startup_packet(sp);
+				alarm(0);
+				pool_signal(SIGALRM, SIG_IGN);
 				return NULL;
 			}
 			sp->user = calloc(1, 1);
@@ -975,6 +1004,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			{
 				pool_error("read_startup_packet: out of memory");
 				pool_free_startup_packet(sp);
+				alarm(0);
+				pool_signal(SIGALRM, SIG_IGN);
 				return NULL;
 			}
 			break;
@@ -982,12 +1013,15 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 		default:
 			pool_error("read_startup_packet: invalid major no: %d", sp->major);
 			pool_free_startup_packet(sp);
+			alarm(0);
+			pool_signal(SIGALRM, SIG_IGN);
 			return NULL;
 	}
 
 	pool_debug("Protocol Major: %d Minor: %d database: %s user: %s", 
 			   sp->major, sp->minor, sp->database, sp->user);
-
+	alarm(0);
+	pool_signal(SIGALRM, SIG_IGN);
 	return sp;
 }
 
@@ -1176,6 +1210,16 @@ static RETSIGTYPE close_idle_connection(int sig)
 	}
 }
 
+/*
+ * signal handler for SIGALRM
+ * 
+ */
+static RETSIGTYPE authentication_timeout(int sig)
+{
+	pool_log("authentication is timeout");
+	exit(1);
+}
+
 /*
  * send frontend exiting messages to all connections.
  * this is called when child life time expires or child max connections expires.
